﻿/*
VERSION:		2.1

USAGE:
	#include "copyObject.as"
	copyObject( source_obj, destination_obj );
	
WHAT IT'S FOR
	Copy the contents of one object into another,
	erasing the destination object's contents first.
	
WHAT IT HANDLES:
	Basic variables	(ie: string, number, boolean)
	Multi-dimensional Arrays
	Normal Objects
	BitmapData
	XMLNode
	
WHAT IT SKIPS:
	Functions
	Most Actionscript classes & components
	MovieClips
	
*/



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 



emptyObject = function( inputObject )
{
	for (var nam in inputObject )
		delete inputObject[nam];
}// emptyObject()



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 



getType = function( inputObject )
{
	var output = "";
	
	// unsupported objects to ignore
	if ( inputObject instanceof Function )
	{
		// this is a function
		output = "function";
	}
	else if ( inputObject instanceof MovieClip )
	{
		// this is a movieClip
		output = "movieClip";
	}
	
	// supported objects to handle
	else if ( inputObject instanceof Array )
	{
		// this is an array
		output = "array";
	}
	else if ( inputObject instanceof XMLNode )
	{
		// this is an object
		output = "xml";
	}
	else if ( inputObject instanceof flash.display.BitmapData )
	{
		// this is an object
		output = "bitmap";
	}
	else if ( inputObject instanceof Object )
	{
		// this is an object
		output = "object";
	}
	else
	{
		// this is a regular variable
		output = "variable";
	}
	return output;	// object, array, string, number, boolean
}// getType()



copyObject = function( srcObj, destObj )
{
	emptyObject( destObj );
	var srcReversed = {};
	for( var childName in srcObj )
		srcReversed[childName] = srcObj[childName];
	for( var childName in srcReversed )
	{
		var thisChild = srcReversed[childName];
		var childType = getType(thisChild);
		
		// handle variables
		if (childType == "variable")
		{
			destObj[childName] = thisChild;
		}
		
		// handle pictures
		if (childType == "bitmap")
		{
			// // create new object
			destObj[childName] = thisChild.clone();
		}
		
		// handle XML
		if (childType == "xml")
		{
			// // create new object
			destObj[childName] = thisChild.cloneNode(true);
		}
		
		// handle Arrays
		if (childType == "array")
		{
			// // create new array
			destObj[childName] = new Array();
			// // set dest, scan array
			copyObject( thisChild, destObj[childName] );
		}
		
		// handle Objects
		if (childType == "object")
		{
			// // create new object
			destObj[childName] = new Object();
			// // set dest, scan object
			copyObject( thisChild, destObj[childName] );
		}
			
	}// END:  for...in
}// copyObject()



